@@ -37,6 +37,7 @@
 #define SETTINGS_KEY_END_SYSTEM_UIDS "end-system-uids"
 #define SETTINGS_KEY_AUTO_LAUNCH_INTERVAL "regular-auto-launch-interval"
 #define SETTINGS_KEY_LAST_RELEASE_CHECK "release-check-time"
+#define SETTINGS_KEY_SHOW_LIVEPATCH_ICON "show-livepatch-status-icon"
 
 #define SETTINGS_UM_SCHEMA "com.ubuntu.update-manager"
 #define SETTINGS_UM_KEY_LAST_LAUNCH "launch-time"
@@ -48,7 +49,6 @@
 #define CRASHREPORT_AUTOREPORT "/var/lib/apport/autoreport"
 #define CRASHREPORT_WHOOPSIE_EXEC "/usr/bin/whoopsie"
 #define CRASHREPORT_WHOOPSIE_SERVICE "whoopsie"
-#define UNICAST_LOCAL_AVAHI_FILE "/run/avahi-daemon/disabled-for-unicast-local"
 #define LIVEPATCH_FILE "/var/snap/canonical-livepatch/current/status"
 
 // security update autolaunch minimal time (12h)
@@ -102,6 +102,7 @@ struct _UpgradeNotifier
    TrayApplet *update;
    TrayApplet *hook;
    TrayApplet *crashreport;
+   TrayApplet *livepatch;
 
    guint update_finished_timer;
 
@@ -113,7 +114,6 @@ struct _UpgradeNotifier
    // these field are "global" (time-wise)
    gboolean hook_pending;
    gboolean crashreport_pending;
-   gboolean unicast_local_avahi_pending;
    gboolean livepatch_pending;
    time_t last_apt_action;
 
