@@ -10,12 +10,17 @@ import sys
 from optparse import OptionParser
 import gettext
 import subprocess
+import distro_info
 
 SYNAPTIC_PINFILE = "/var/lib/synaptic/preferences"
 DISTRO = subprocess.check_output(
     ["lsb_release", "-c", "-s"],
     universal_newlines=True).strip()
 
+ESM_INFRA_ORIGIN = "UbuntuESM"
+ESM_APPS_ORIGIN = "UbuntuESMApps"
+ESM_ORIGINS = (ESM_INFRA_ORIGIN, ESM_APPS_ORIGIN)
+
 
 def _(msg):
     return gettext.dgettext("update-notifier", msg)
@@ -45,7 +50,8 @@ def saveDistUpgrade(cache, depcache):
 def isSecurityUpgrade(ver):
     " check if the given version is a security update (or masks one) "
     security_pockets = [("Ubuntu", "%s-security" % DISTRO),
-                        ("UbuntuESM", "%s-security" % DISTRO),
+                        (ESM_INFRA_ORIGIN, "%s-infra-security" % DISTRO),
+                        (ESM_APPS_ORIGIN, "%s-apps-security" % DISTRO),
                         ("gNewSense", "%s-security" % DISTRO),
                         ("Debian", "%s-updates" % DISTRO)]
     for (file, index) in ver.file_list:
@@ -55,14 +61,22 @@ def isSecurityUpgrade(ver):
     return False
 
 
-def isESMUpgrade(ver):
+def _isESMUpgrade(ver, esm_origin):
     " check if the given version is a security update (or masks one) "
     for (file, index) in ver.file_list:
-        if file.origin == "UbuntuESM" and file.archive.startswith(DISTRO):
+        if file.origin == esm_origin and file.archive.startswith(DISTRO):
             return True
     return False
 
 
+def isESMAppsUpgrade(ver):
+    return _isESMUpgrade(ver, esm_origin=ESM_APPS_ORIGIN)
+
+
+def isESMInfraUpgrade(ver):
+    return _isESMUpgrade(ver, esm_origin=ESM_INFRA_ORIGIN)
+
+
 def write_package_names(outstream, cache, depcache):
     " write out package names that change to outstream "
     pkgs = [pkg for pkg in cache.packages if depcache.marked_install(pkg)
@@ -70,78 +84,140 @@ def write_package_names(outstream, cache, depcache):
     outstream.write("\n".join([p.name for p in pkgs]))
 
 
+def is_esm_distro():
+    ubuntu_distro = distro_info.UbuntuDistroInfo()
+
+    is_esm_supported = bool(
+        DISTRO in ubuntu_distro.supported_esm()
+    )
+
+    is_not_currently_supported = bool(
+        DISTRO in ubuntu_distro.unsupported()
+    )
+
+    return is_esm_supported and is_not_currently_supported
+
+
+def is_lts_distro():
+    return distro_info.UbuntuDistroInfo().is_lts(DISTRO)
+
+
+def _output_esm_package_count(outstream, service_type, esm_pkg_count):
+    if esm_pkg_count > 0:
+        outstream.write("\n")
+        outstream.write(gettext.dngettext("update-notifier",
+                                          "%d of these updates "
+                                          "is an UA %s: ESM "
+                                          "security update.",
+                                          "%d of these updates "
+                                          "are UA %s: ESM "
+                                          "security updates.",
+                                          esm_pkg_count) %
+                        (esm_pkg_count, service_type))
+
+
+def _output_esm_package_alert(
+    outstream, service_type, disabled_pkg_count
+):
+    outstream.write("\n")
+    if disabled_pkg_count > 0:
+        outstream.write("\n")
+        outstream.write(gettext.dngettext("update-notifier",
+                                          "%i additional security "
+                                          "update can be applied "
+                                          "with UA %s: ESM\nLearn "
+                                          "more about enabling UA "
+                                          "%s: ESM service at "
+                                          "https://ubuntu.com/esm",
+                                          "%i additional security "
+                                          "updates can be applied "
+                                          "with UA %s: ESM\nLearn "
+                                          "more about enabling UA "
+                                          "%s: ESM service at "
+                                          "https://ubuntu.com/esm",
+                                          disabled_pkg_count) %
+                        (disabled_pkg_count, service_type, service_type))
+    else:
+        outstream.write("\n")
+        outstream.write(gettext.dgettext("update-notifier",
+                                         "Enable UA %s: ESM to "
+                                         "receive additional future "
+                                         "security updates.") %
+                        service_type)
+
+        outstream.write("\n")
+        outstream.write(
+            gettext.dgettext("update-notifier",
+                             "See https://ubuntu.com/security/esm "
+                             "or run: sudo ua status")
+        )
+
+
 def write_human_readable_summary(outstream, upgrades, security_updates,
-                                 esm_updates, have_esm, disabled_esm_updates):
+                                 esm_infra_updates, esm_apps_updates,
+                                 have_esm_infra, have_esm_apps,
+                                 disabled_esm_infra_updates,
+                                 disabled_esm_apps_updates):
+
+    esm_distro = is_esm_distro()
+    lts_distro = is_lts_distro()
     " write out human summary summary to outstream "
-    if have_esm is not None:
-        if have_esm:
+    if have_esm_infra is not None and esm_distro:
+        if have_esm_infra:
             outstream.write(gettext.dgettext("update-notifier",
-                                             "UA Infrastructure Extended "
+                                             "UA Infra: Extended "
                                              "Security Maintenance (ESM) is "
                                              "enabled."))
         else:
             outstream.write(gettext.dgettext("update-notifier",
-                                             "UA Infrastructure Extended "
+                                             "UA Infra: Extended "
                                              "Security Maintenance (ESM) is "
                                              "not enabled."))
-        outstream.write("\n\n")
+        if upgrades > 0:
+            outstream.write("\n\n")
+    if upgrades > 0:
+        outstream.write(
+            gettext.dngettext("update-notifier",
+                              "%i update can be applied immediately.",
+                              "%i updates can be applied immediately.",
+                              upgrades) % upgrades
+        )
 
-    outstream.write(gettext.dngettext("update-notifier",
-                                      "%i update can be installed "
-                                      "immediately.",
-                                      "%i updates can be installed "
-                                      "immediately.",
-                                      upgrades) % upgrades)
-    outstream.write("\n")
-    if esm_updates > 0:
-        outstream.write(gettext.dngettext("update-notifier",
-                                          "%i of these updates is provided "
-                                          "through UA Infrastructure ESM.",
-                                          "%i of these updates are "
-                                          "provided through UA "
-                                          "Infrastructure ESM.",
-                                          esm_updates) %
-                        esm_updates)
+    _output_esm_package_count(
+        outstream, service_type="Infra", esm_pkg_count=esm_infra_updates)
+    _output_esm_package_count(
+        outstream, service_type="Apps", esm_pkg_count=esm_apps_updates)
+
+    if security_updates > 0:
         outstream.write("\n")
-    outstream.write(gettext.dngettext("update-notifier",
-                                      "%i of these updates is a "
-                                      "security update.",
-                                      "%i of these updates are "
-                                      "security updates.",
-                                      security_updates) %
-                    security_updates)
-    if upgrades > 0 or security_updates > 0 or esm_updates > 0:
+        outstream.write(gettext.dngettext("update-notifier",
+                                          "%i of these updates is a "
+                                          "standard security update.",
+                                          "%i of these updates are "
+                                          "standard security updates.",
+                                          security_updates) %
+                        security_updates)
+
+    if any([upgrades, security_updates, esm_infra_updates, esm_apps_updates]):
         outstream.write("\n")
         outstream.write(gettext.dgettext("update-notifier",
                                          "To see these additional updates "
                                          "run: apt list --upgradable"))
-    if have_esm is not None and not have_esm:
-        outstream.write("\n")
-        if disabled_esm_updates > 0:
-            outstream.write("\n")
-            outstream.write(gettext.dngettext("update-notifier",
-                                              "Enable UA Infrastructure ESM "
-                                              "to receive %i additional "
-                                              "security update.",
-                                              "Enable UA Infrastructure ESM "
-                                              "to receive %i additional "
-                                              "security updates.",
-                                              disabled_esm_updates) %
-                            disabled_esm_updates)
-        else:
-            outstream.write("\n")
-            outstream.write(gettext.dgettext("update-notifier",
-                                             "Enable UA Infrastructure ESM to "
-                                             "receive additional future "
-                                             "security updates."))
-        outstream.write("\n")
-        outstream.write(gettext.dgettext("update-notifier",
-                                         "See https://ubuntu.com/esm "
-                                         "or run: sudo ua status"))
+
+    if have_esm_apps is not None and not have_esm_apps and lts_distro:
+        _output_esm_package_alert(
+            outstream, service_type="Apps",
+            disabled_pkg_count=disabled_esm_apps_updates)
+
+    if have_esm_infra is not None and not have_esm_infra and esm_distro:
+        _output_esm_package_alert(
+            outstream, service_type="Infra",
+            disabled_pkg_count=disabled_esm_infra_updates)
+
     outstream.write("\n")
 
 
-def has_disabled_esm_security_update(depcache, pkg):
+def has_disabled_esm_security_update(depcache, pkg, esm_origin):
     " check if we have a disabled ESM security update "
     inst_ver = pkg.current_ver
     if not inst_ver:
@@ -152,12 +228,39 @@ def has_disabled_esm_security_update(depcache, pkg):
             break
 
         for (file, index) in ver.file_list:
-            if (file.origin == "UbuntuESM" and file.archive.startswith(DISTRO)
+            if (file.origin == esm_origin and file.archive.startswith(DISTRO)
                     and depcache.policy.get_priority(file) == -32768):
                 return True
     return False
 
 
+def has_disabled_esm_apps_security_update(depcache, pkg):
+    return has_disabled_esm_security_update(depcache, pkg, ESM_APPS_ORIGIN)
+
+
+def has_disabled_esm_infra_security_update(depcache, pkg):
+    return has_disabled_esm_security_update(depcache, pkg, ESM_INFRA_ORIGIN)
+
+
+def has_esm_service(cache, depcache, esm_origin):
+    have_esm = None
+    for file in cache.file_list:
+        origin = file.origin
+        if origin == esm_origin and file.archive.startswith(DISTRO):
+            # In case of multiple ESM repos, one enabled is sufficient.
+            if depcache.policy.get_priority(file) == -32768:
+                # We found a disabled ESM repository, but we'll only count
+                # ESM as disabled here if we have not found any other ESM
+                # repo, so one ESM repo being enabled means ESM is enabled.
+                if have_esm is None:
+                    have_esm = False
+            else:
+                have_esm = True
+                break
+
+    return have_esm
+
+
 def init():
     " init the system, be nice "
     # FIXME: do a ionice here too?
@@ -200,31 +303,26 @@ def run(options=None):
 
     # Check if we have ESM enabled or disabled; and if it exists in the
     # first place.
-    have_esm = None        # None == does not exist
-    for file in cache.file_list:
-        if file.origin == "UbuntuESM" and file.archive.startswith(DISTRO):
-            # In case of multiple ESM repos, one enabled is sufficient.
-            if depcache.policy.get_priority(file) == -32768:
-                # We found a disabled ESM repository, but we'll only count
-                # ESM as disabled here if we have not found any other ESM
-                # repo, so one ESM repo being enabled means ESM is enabled.
-                if have_esm is None:
-                    have_esm = False
-            else:
-                have_esm = True
-                break
+    have_esm_infra = has_esm_service(
+        cache, depcache, esm_origin=ESM_INFRA_ORIGIN)
+    have_esm_apps = has_esm_service(
+        cache, depcache, esm_origin=ESM_APPS_ORIGIN)
 
     # analyze the ugprade
     upgrades = 0
     security_updates = 0
-    esm_updates = 0
-    disabled_esm_updates = 0
+    esm_apps_updates = 0
+    esm_infra_updates = 0
+    disabled_esm_apps_updates = 0
+    disabled_esm_infra_updates = 0
 
     # we need another cache that has more pkg details
     with apt.Cache() as aptcache:
         for pkg in cache.packages:
-            if has_disabled_esm_security_update(depcache, pkg):
-                disabled_esm_updates += 1
+            if has_disabled_esm_apps_security_update(depcache, pkg):
+                disabled_esm_apps_updates += 1
+            if has_disabled_esm_infra_security_update(depcache, pkg):
+                disabled_esm_infra_updates += 1
 
             # skip packages that are not marked upgraded/installed
             if not (depcache.marked_install(pkg)
@@ -238,10 +336,14 @@ def run(options=None):
                 continue
             # check for security upgrades
             if isSecurityUpgrade(cand_ver):
-                if isESMUpgrade(cand_ver):
-                    esm_updates += 1
+                if have_esm_apps and isESMAppsUpgrade(cand_ver):
+                    esm_apps_updates += 1
+                elif have_esm_infra and isESMInfraUpgrade(cand_ver):
+                    esm_infra_updates += 1
+                else:
+                    security_updates += 1
+
                 upgrades += 1
-                security_updates += 1
                 continue
 
             # check to see if the update is a phased one
@@ -265,9 +367,11 @@ def run(options=None):
                         and apt_pkg.version_compare(ver.ver_str,
                                                     inst_ver.ver_str) <= 0):
                     continue
-                if isESMUpgrade(ver):
-                    esm_updates += 1
-                if isSecurityUpgrade(ver):
+                if have_esm_apps and isESMAppsUpgrade(cand_ver):
+                    esm_apps_updates += 1
+                elif have_esm_infra and isESMInfraUpgrade(cand_ver):
+                    esm_infra_updates += 1
+                elif isSecurityUpgrade(ver):
                     security_updates += 1
                     break
 
@@ -276,8 +380,10 @@ def run(options=None):
         write_package_names(sys.stderr, cache, depcache)
     elif options and options.readable_output:
         write_human_readable_summary(sys.stdout, upgrades, security_updates,
-                                     esm_updates, have_esm,
-                                     disabled_esm_updates)
+                                     esm_infra_updates, esm_apps_updates,
+                                     have_esm_infra, have_esm_apps,
+                                     disabled_esm_infra_updates,
+                                     disabled_esm_apps_updates)
     else:
         # print the number of regular upgrades and the number of
         # security upgrades
